@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _CONTEXT_H_
 #define _CONTEXT_H_
@@ -10,82 +10,107 @@
 #include <map>
 #include <string>
 #include "CGNSOptions.h"
-#include "meshPartitionOptions.h"
 
 #define NUM_SOLVERS 10
 
 class GamePad;
 
-// The interface-independent context.
-
 struct contextMeshOptions {
-  int draw, changed, light, lightTwoSide, lightLines, pointType;
-  int points, lines, triangles, quadrangles, tetrahedra, hexahedra, prisms, pyramids;
-  int surfacesEdges, surfacesFaces, volumesEdges, volumesFaces, numSubEdges;
-  int pointsNum, linesNum, surfacesNum, volumesNum, qualityType, labelType;
-  int optimize, optimizeNetgen, optimizeLloyd, smoothCrossField, refineSteps;
-  int remove4triangles;
-  double normals, tangents, explode, angleSmoothNormals, allowSwapEdgeAngle;
-  double mshFileVersion, mshFilePartitioned, pointSize, lineWidth;
+  // mesh algorithms
+  int optimize, optimizeNetgen, smoothCrossField, refineSteps;
+  double lcFactor, randFactor, randFactor3d, lcIntegrationPrecision;
+  double optimizeThreshold, normals, tangents, explode, angleSmoothNormals;
+  double allowSwapEdgeAngle;
   double qualityInf, qualitySup, radiusInf, radiusSup;
-  double scalingFactor, lcFactor, randFactor, lcIntegrationPrecision;
   double lcMin, lcMax, toleranceEdgeLength, toleranceInitialDelaunay;
   double anisoMax, smoothRatio;
   int lcFromPoints, lcFromCurvature, lcExtendFromBoundary;
-  int dual, voronoi, drawSkinOnly, colorCarousel, labelSampling;
-  int fileFormat, nbSmoothing, algo2d, algo3d, algoSubdivide;
-  int algoRecombine, recombineAll, recombine3DAll, flexibleTransfinite;
-  //-- for recombination test (amaury) --
-    int doRecombinationTest, recombinationTestStart;
-    int recombinationTestNoGreedyStrat, recombinationTestNewStrat;
-    int nProc, nbProc;
-    std::string recTestName;
-  //-------------------------------------
-  int remeshParam, remeshAlgo;
-  int order, secondOrderLinear, secondOrderIncomplete;
-  int secondOrderExperimental, meshOnlyVisible;
-  int minCircPoints, minCurvPoints;
+  int nbSmoothing, algo2d, algo3d, algoSubdivide, oldRefinement;
+  int algoRecombine, recombineAll, recombine3DAll, recombine3DLevel;
+  int recombine3DConformity, flexibleTransfinite, remeshParam, remeshAlgo;
+  int order, secondOrderLinear, secondOrderIncomplete, secondOrderExperimental;
+  int meshOnlyVisible, minCircPoints, minCurvPoints;
   int hoOptimize, hoNLayers, hoOptPrimSurfMesh;
   double hoThresholdMin, hoThresholdMax, hoPoissonRatio;
-  int saveAll, saveTri, saveGroupsOfNodes, binary, bdfFieldFormat, saveParametric;
-  int smoothNormals, reverseAllNormals, zoneDefinition, clip;
-  int saveElementTagType;
-  int switchElementTags;
-  int multiplePasses;
-  int cgnsImportOrder;
-  std::map<int,int> algo2d_per_face;
-  std::map<int,int> curvature_control_per_face;
-  int bunin;
-  int ignorePartBound;
+  std::map<int, int> algo2dPerFace;
+  std::map<int, int> curvatureControlPerFace;
   int NewtonConvergenceTestXYZ;
+  int ignorePeriodicity, boundaryLayerFanPoints;
+  int maxNumThreads1D, maxNumThreads2D, maxNumThreads3D;
+  double angleToleranceFacetOverlap;
+  // mesh IO
+  int fileFormat;
+  double mshFileVersion, scalingFactor;
+  int saveAll, saveTri, saveGroupsOfNodes, binary, bdfFieldFormat;
+  int saveParametric, saveTopology, zoneDefinition;
+  int saveElementTagType, switchElementTags;
+  int cgnsImportOrder, cgnsConstructTopology;
+  int preserveNumberingMsh2;
+  // partitioning
+  int numPartitions, partitionCreateTopology, partitionCreateGhostCells;
+  int partitionCreatePhysicals, partitionSplitMeshFiles,
+    partitionSaveTopologyFile;
+  int partitionTriWeight, partitionQuaWeight, partitionTetWeight,
+    partitionHexWeight;
+  int partitionPriWeight, partitionPyrWeight, partitionTrihWeight;
+  int partitionOldStyleMsh2;
+  int metisAlgorithm, metisEdgeMatching, metisRefinementAlgorithm;
+  // recombination
+  int doRecombinationTest, recombinationTestStart;
+  int recombinationTestNoGreedyStrat, recombinationTestNewStrat, nProc, nbProc;
+  std::string recTestName;
+  // mesh display
+  int draw, changed, light, lightTwoSide, lightLines, pointType;
+  int points, lines, triangles, quadrangles, tetrahedra, hexahedra, prisms;
+  int pyramids, trihedra;
+  int surfacesEdges, surfacesFaces, volumesEdges, volumesFaces, numSubEdges;
+  int pointsNum, linesNum, surfacesNum, volumesNum, qualityType, labelType;
+  double pointSize, lineWidth;
+  int dual, voronoi, drawSkinOnly, colorCarousel, labelSampling;
+  int smoothNormals, clip;
 };
 
 struct contextGeometryOptions {
-  int draw, light, lightTwoSide, points, lines, surfaces, volumes;
-  int pointsNum, linesNum, surfacesNum, volumesNum, labelType;
-  double pointSize, lineWidth, selectedPointSize, selectedLineWidth;
-  int pointType, lineType, surfaceType, numSubEdges;
+  // geometry algorithms
   int oldCircle, oldNewreg, oldRuledSurface;
   int extrudeSplinePoints, extrudeReturnLateral;
-  double normals, tangents, scalingFactor;
-  int autoCoherence, highlightOrphans, clip, useTransform;
-  double tolerance, snap[3], transform[3][3], offset[3];
-  int occFixDegenerated, occFixSmallEdges, occFixSmallFaces;
-  int occSewFaces, occConnectFaces;
+  int autoCoherence;
+  double tolerance, toleranceBoolean, snap[3], transform[3][3], offset[3];
+  int occAutoFix, occFixDegenerated, occFixSmallEdges, occFixSmallFaces;
+  int occSewFaces, occParallel, occBooleanPreserveNumbering;
+  int occDisableSTL;
   double occScaling;
-  int copyMeshingMethod, copyDisplayAttributes, exactExtrusion;
+  std::string occTargetUnit;
+  int copyMeshingMethod, exactExtrusion;
   int matchGeomAndMesh;
+  double matchMeshScaleFactor;
+  double matchMeshTolerance;
   int hideCompounds, orientedPhysicals;
+  int reparamOnFaceRobust;
+  // geometry display
+  int draw, light, lightTwoSide, points, curves, surfaces, volumes;
+  int pointsNum, curvesNum, surfacesNum, volumesNum, labelType;
+  double pointSize, curveWidth, selectedPointSize, selectedCurveWidth;
+  int pointType, curveType, surfaceType, numSubEdges;
+  double normals, tangents, scalingFactor;
+  int highlightOrphans, clip, useTransform;
+  int doubleClickedEntityTag;
+  std::string doubleClickedPointCommand, doubleClickedCurveCommand;
+  std::string doubleClickedSurfaceCommand, doubleClickedVolumeCommand;
 };
 
+// The interface-independent context.
 class CTX {
- private:
+private:
   static CTX *_instance;
- public:
+
+public:
   CTX();
   ~CTX();
   static CTX *instance();
  public:
+  // for debug purposes only, i.e. JF and CG personal use
+  int debugSurface;
   // files on the command line and various file names
   std::vector<std::string> files;
   std::string bgmFileName, outputFileName, defaultFileName, tmpFileName;
@@ -100,9 +125,7 @@ class CTX {
   // create mesh statistics report (0: do nothing, 1: create, 2: append)
   int createAppendMeshStatReport;
   // should we launch a solver at startup?
-  int launchSolverAtStartup ;
-  // solver to use with ONELAB2
-  int solverToRun ;
+  int launchSolverAtStartup;
   // save session/option file on exit?
   int sessionSave, optionsSave;
   // ask confirmation when overwriting files?
@@ -124,8 +147,10 @@ class CTX {
   // show tootips in the GUI?
   int tooltips;
   // position and size of various windows in the GUI
-  int glPosition[2], glSize[2], msgSize, menuPosition[2], menuSize[2], detachedMenu;
-  int optPosition[2], visPosition[2], hotPosition[2], clipPosition[2], manipPosition[2];
+  int glPosition[2], glSize[2], msgSize, menuPosition[2], menuSize[2],
+    detachedMenu;
+  int optPosition[2], visPosition[2], hotPosition[2], clipPosition[2],
+    manipPosition[2];
   int statPosition[2], ctxPosition[2];
   int pluginPosition[2], pluginSize[2], fieldPosition[2], fieldSize[2];
   int fileChooserPosition[2], extraPosition[2], extraSize[2];
@@ -133,14 +158,18 @@ class CTX {
   int systemMenuBar;
   // use high-resolution opengl graphics (retina Macs)
   int highResolutionGraphics;
-  // batch mode (-4: lua session, -3: server daemon, -2: check coherence, -1: write
-  // geo, 0: full gfx, 1: 1D mesh, 2: 2D mesh, 3: 3D mesh, 4: adapt
-  // mesh, 5: refine mesh)
+  // batch mode (-4: lua session, -3: server daemon, -2: check coherence, -1:
+  // write geo, 0: full gfx, 1: 1D mesh, 2: 2D mesh, 3: 3D mesh, 4: adapt mesh,
+  // 5: refine mesh, 6: reclassify mesh)
   int batch;
   // batch operations to apply after meshing (1: partition mesh)
   int batchAfterMesh;
+  // mesh discrete faces / edges
+  int meshDiscrete;
   // initial menu (0: automatic, 1: geom, 2: mesh, 3: solver, 4: post)
   int initialContext;
+  // show some windows on startup?
+  int showOptionsOnStartup, showMessagesOnStartup;
   // never popup dialogs in scripts (use default values instead)?
   int noPopup;
   // make all windows "non modal"?
@@ -161,13 +190,13 @@ class CTX {
   double min[3], max[3];
   // "center of mass" of the current geometry, used for graphics only
   double cg[3];
- // characteristic length for the whole problem (never used in mesh
+  // characteristic length for the whole problem (never used in mesh
   // generation ->only for geo/post)
   double lc;
   // double buffer/antialias/stereo graphics?
-  int db, antialiasing, stereo, camera ;
+  int db, antialiasing, stereo, camera;
   bool fileread;
-  double eye_sep_ratio,focallength_ratio,camera_aperture;
+  double eye_sep_ratio, focallength_ratio, camera_aperture;
   // orthogonal projection?
   int ortho;
   // draw the bounding boxes and the rot center?
@@ -224,8 +253,14 @@ class CTX {
   int forcedBBox;
   // enable selection/hover/picking using the mouse
   int mouseSelection, mouseHoverMeshes, pickElements;
+  // invert sense of mouse wheel zoom
+  int mouseInvertZoom;
   // disable some warnings for expert users?
   int expertMode;
+#if defined(HAVE_VISUDEV)
+  // Enable heavy visualization capabilities (for development purpose)
+  int heavyVisu;
+#endif
   // dynamic: equal to 1 while gmsh is printing
   int printing;
   // hide all unselected entities?
@@ -237,59 +272,58 @@ class CTX {
   contextGeometryOptions geom;
   // mesh options
   contextMeshOptions mesh;
-  // FIXME: putting these in the mesh struct (where they belong) causes
-  // an LNK1179 error ("duplicate COMDAT") with MSVC...
-  meshPartitionOptions partitionOptions;
   CGNSOptions cgnsOptions;
   // post processing options
-  struct{
+  struct {
     int draw, link, horizontalScales;
     int smooth, animCycle, animStep, combineTime, combineRemoveOrig;
-    int fileFormat, plugins, forceNodeData;
+    int fileFormat, plugins, forceNodeData, forceElementData;
+    int saveMesh, saveInterpolationMatrices;
     double animDelay;
-    std::string graphPointCommand;
-    double graphPointX, graphPointY;
-  }post;
+    std::string doubleClickedGraphPointCommand;
+    double doubleClickedGraphPointX, doubleClickedGraphPointY;
+    int doubleClickedView;
+  } post;
   // solver options
-  struct{
+  struct {
     int plugins, listen;
     double timeout;
     std::string socketName, pythonInterpreter, octaveInterpreter;
-    std::string name[NUM_SOLVERS], executable[NUM_SOLVERS], remoteLogin[NUM_SOLVERS];
-    int autoSaveDatabase, autoArchiveOutputFiles, autoMesh, autoMergeFile;
+    std::string name[NUM_SOLVERS], extension[NUM_SOLVERS];
+    std::string executable[NUM_SOLVERS], remoteLogin[NUM_SOLVERS];
+    int autoSaveDatabase, autoLoadDatabase;
+    int autoArchiveOutputFiles, autoMesh, autoMergeFile;
     int autoShowViews, autoShowLastStep, autoCheck, showInvisibleParameters;
-  }solver;
-  // onelab options
-  struct{
-    short listen_port;
-  }onelab;
+  } solver;
   // print options
-  struct{
+  struct {
     int fileFormat, epsQuality, epsCompress, epsPS3Shading;
     int epsOcclusionCulling, epsBestRoot;
     double epsLineWidthFactor, epsPointSizeFactor;
     int jpegQuality, jpegSmoothing, geoLabels, geoOnlyPhysicals;
     int text, texAsEquation;
     int gifDither, gifSort, gifInterlace, gifTransparent;
-    int posElementary, posElement, posGamma, posEta, posRho, posDisto;
+    int posElementary, posElement, posGamma, posEta, posSICN, posSIGE, posDisto;
     int compositeWindows, deleteTmpFiles, background;
     int width, height;
     double parameter, parameterFirst, parameterLast, parameterSteps;
     int pgfTwoDim, pgfExportAxis, pgfHorizBar;
     std::string parameterCommand;
+    int x3dCompatibility, x3dRemoveInnerBorders;
+    double x3dPrecision, x3dTransparency;
   } print;
   // color options
-  struct{
+  struct {
     unsigned int bg, bgGrad, fg, text, axes, smallAxes;
     unsigned int ambientLight[6], diffuseLight[6], specularLight[6];
-    struct{
-      unsigned int point, line, surface, volume;
+    struct {
+      unsigned int point, curve, surface, volume;
       unsigned int selection, highlight[3], projection;
       unsigned int tangents, normals;
     } geom;
-    struct{
-      unsigned int vertex, vertexSup, line, triangle, quadrangle;
-      unsigned int tetrahedron, hexahedron, prism, pyramid;
+    struct {
+      unsigned int node, nodeSup, line, triangle, quadrangle;
+      unsigned int tetrahedron, hexahedron, prism, pyramid, trihedron;
       unsigned int carousel[20];
       unsigned int tangents, normals;
     } mesh;
