@@ -1,17 +1,17 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "GModel.h"
-#include "PView.h"
 #include "GmshDefines.h"
 #include "StringUtils.h"
 #include "Context.h"
 #include "Options.h"
 #include "OpenFile.h"
+#include "CreateFile.h"
 #include "OS.h"
 
 #if defined(HAVE_OPENGL)
@@ -21,110 +21,154 @@
 #if defined(HAVE_FLTK)
 #include "FlGui.h"
 #include "graphicWindow.h"
+#include "openglWindow.h"
 #include "gl2ps.h"
 #include "gl2gif.h"
 #include "gl2jpeg.h"
 #include "gl2png.h"
 #include "gl2ppm.h"
 #include "gl2yuv.h"
+#include "gl2pgf.h"
 #endif
 
-int GetFileFormatFromExtension(const std::string &ext)
+int GetFileFormatFromExtension(const std::string &ext, double *version)
 {
-  if     (ext == ".geo")  return FORMAT_GEO;
-  else if(ext == ".msh")  return FORMAT_MSH;
-  else if(ext == ".pos")  return FORMAT_POS;
-  else if(ext == ".opt")  return FORMAT_OPT;
-  else if(ext == ".unv")  return FORMAT_UNV;
-  else if(ext == ".vtk")  return FORMAT_VTK;
-  else if(ext == ".txt")  return FORMAT_TXT;
-  else if(ext == ".stl")  return FORMAT_STL;
-  else if(ext == ".cgns") return FORMAT_CGNS;
-  else if(ext == ".med")  return FORMAT_MED;
-  else if(ext == ".rmed") return FORMAT_RMED;
-  else if(ext == ".ir3")  return FORMAT_IR3;
-  else if(ext == ".mesh") return FORMAT_MESH;
-  else if(ext == ".mail") return FORMAT_MAIL;
-  else if(ext == ".bdf")  return FORMAT_BDF;
-  else if(ext == ".diff") return FORMAT_DIFF;
-  else if(ext == ".inp")  return FORMAT_INP;
-  else if(ext == ".celum")return FORMAT_CELUM;
-  else if(ext == ".su2")  return FORMAT_SU2;
-  else if(ext == ".nas")  return FORMAT_BDF;
-  else if(ext == ".p3d")  return FORMAT_P3D;
-  else if(ext == ".wrl")  return FORMAT_VRML;
-  else if(ext == ".vrml") return FORMAT_VRML;
-  else if(ext == ".ply2") return FORMAT_PLY2;
-  else if(ext == ".gif")  return FORMAT_GIF;
-  else if(ext == ".jpg")  return FORMAT_JPEG;
-  else if(ext == ".jpeg") return FORMAT_JPEG;
-  else if(ext == ".mpg")  return FORMAT_MPEG;
-  else if(ext == ".mpeg") return FORMAT_MPEG;
-  else if(ext == ".png")  return FORMAT_PNG;
-  else if(ext == ".ps")   return FORMAT_PS;
-  else if(ext == ".eps")  return FORMAT_EPS;
-  else if(ext == ".pdf")  return FORMAT_PDF;
-  else if(ext == ".tex")  return FORMAT_TEX;
-  else if(ext == ".svg")  return FORMAT_SVG;
-  else if(ext == ".ppm")  return FORMAT_PPM;
-  else if(ext == ".yuv")  return FORMAT_YUV;
-  else if(ext == ".brep") return FORMAT_BREP;
-  else if(ext == ".step") return FORMAT_STEP;
-  else if(ext == ".stp")  return FORMAT_STEP;
-  else if(ext == ".iges") return FORMAT_IGES;
-  else if(ext == ".igs")  return FORMAT_IGES;
-  else                           return -1;
+  if(ext == ".geo_unrolled")  return FORMAT_GEO;
+  else if(ext == ".auto")     return FORMAT_AUTO;
+  else if(ext == ".msh")      return FORMAT_MSH;
+  else if(ext == ".msh1")     { if(version) *version = 1.0; return FORMAT_MSH; }
+  else if(ext == ".msh2")     { if(version) *version = 2.2; return FORMAT_MSH; }
+  else if(ext == ".msh3")     { if(version) *version = 3.0; return FORMAT_MSH; }
+  else if(ext == ".msh4")     { if(version) *version = 4.0; return FORMAT_MSH; }
+  else if(ext == ".x3d")      return FORMAT_X3D;
+  else if(ext == ".pos")      return FORMAT_POS;
+  else if(ext == ".pvtu")     return FORMAT_PVTU;
+  else if(ext == ".opt")      return FORMAT_OPT;
+  else if(ext == ".unv")      return FORMAT_UNV;
+  else if(ext == ".vtk")      return FORMAT_VTK;
+  else if(ext == ".m")        return FORMAT_MATLAB;
+  else if(ext == ".dat")      return FORMAT_TOCHNOG;
+  else if(ext == ".txt")      return FORMAT_TXT;
+  else if(ext == ".stl")      return FORMAT_STL;
+  else if(ext == ".cgns")     return FORMAT_CGNS;
+  else if(ext == ".med")      return FORMAT_MED;
+  else if(ext == ".rmed")     return FORMAT_RMED;
+  else if(ext == ".ir3")      return FORMAT_IR3;
+  else if(ext == ".mesh")     return FORMAT_MESH;
+  else if(ext == ".mail")     return FORMAT_MAIL;
+  else if(ext == ".bdf")      return FORMAT_BDF;
+  else if(ext == ".diff")     return FORMAT_DIFF;
+  else if(ext == ".inp")      return FORMAT_INP;
+  else if(ext == ".key")      return FORMAT_KEY;
+  else if(ext == ".celum")    return FORMAT_CELUM;
+  else if(ext == ".su2")      return FORMAT_SU2;
+  else if(ext == ".nas")      return FORMAT_BDF;
+  else if(ext == ".p3d")      return FORMAT_P3D;
+  else if(ext == ".wrl")      return FORMAT_VRML;
+  else if(ext == ".vrml")     return FORMAT_VRML;
+  else if(ext == ".ply2")     return FORMAT_PLY2;
+  else if(ext == ".gif")      return FORMAT_GIF;
+  else if(ext == ".jpg")      return FORMAT_JPEG;
+  else if(ext == ".jpeg")     return FORMAT_JPEG;
+  else if(ext == ".mpg")      return FORMAT_MPEG;
+  else if(ext == ".mpeg")     return FORMAT_MPEG;
+  else if(ext == ".png")      return FORMAT_PNG;
+  else if(ext == ".pgf")      return FORMAT_PGF;
+  else if(ext == ".ps")       return FORMAT_PS;
+  else if(ext == ".eps")      return FORMAT_EPS;
+  else if(ext == ".pdf")      return FORMAT_PDF;
+  else if(ext == ".tex")      return FORMAT_TEX;
+  else if(ext == ".svg")      return FORMAT_SVG;
+  else if(ext == ".tikz")     return FORMAT_TIKZ;
+  else if(ext == ".ppm")      return FORMAT_PPM;
+  else if(ext == ".yuv")      return FORMAT_YUV;
+  else if(ext == ".brep")     return FORMAT_BREP;
+  else if(ext == ".step")     return FORMAT_STEP;
+  else if(ext == ".stp")      return FORMAT_STEP;
+  else if(ext == ".iges")     return FORMAT_IGES;
+  else if(ext == ".igs")      return FORMAT_IGES;
+  else if(ext == ".neu")      return FORMAT_NEU;
+  else                        return -1;
 }
 
-int GuessFileFormatFromFileName(const std::string &fileName)
+int GuessFileFormatFromFileName(const std::string &fileName, double *version)
 {
   std::string ext = SplitFileName(fileName)[2];
-  return GetFileFormatFromExtension(ext);
+  return GetFileFormatFromExtension(ext, version);
+}
+
+std::string GetDefaultFileExtension(int format, bool onlyMeshFormats)
+{
+  std::string name;
+  bool mesh = false;
+  switch(format){
+  case FORMAT_GEO:     name = ".geo_unrolled"; break;
+  case FORMAT_MSH:     name = ".msh"; mesh = true; break;
+  case FORMAT_POS:     name = ".pos"; break;
+  case FORMAT_X3D:     name = ".x3d"; mesh = true; break;
+  case FORMAT_PVTU:    name = ".pvtu"; break;
+  case FORMAT_OPT:     name = ".opt"; break;
+  case FORMAT_UNV:     name = ".unv"; mesh = true; break;
+  case FORMAT_VTK:     name = ".vtk"; mesh = true; break;
+  case FORMAT_MATLAB:  name = ".m"; mesh = true; break;
+  case FORMAT_TOCHNOG: name = ".dat"; mesh = true; break;
+  case FORMAT_STL:     name = ".stl"; mesh = true; break;
+  case FORMAT_CGNS:    name = ".cgns"; mesh = true; break;
+  case FORMAT_MED:     name = ".med"; mesh = true; break;
+  case FORMAT_RMED:    name = ".rmed"; break;
+  case FORMAT_IR3:     name = ".ir3"; mesh = true; break;
+  case FORMAT_MESH:    name = ".mesh"; mesh = true; break;
+  case FORMAT_MAIL:    name = ".mail"; mesh = true; break;
+  case FORMAT_BDF:     name = ".bdf"; mesh = true; break;
+  case FORMAT_DIFF:    name = ".diff"; mesh = true; break;
+  case FORMAT_INP:     name = ".inp"; mesh = true; break;
+  case FORMAT_KEY:     name = ".key"; mesh = true; break;
+  case FORMAT_CELUM:   name = ".celum"; mesh = true; break;
+  case FORMAT_SU2:     name = ".su2"; mesh = true; break;
+  case FORMAT_P3D:     name = ".p3d"; mesh = true; break;
+  case FORMAT_VRML:    name = ".wrl"; mesh = true; break;
+  case FORMAT_PLY2:    name = ".ply2"; mesh = true; break;
+  case FORMAT_GIF:     name = ".gif"; break;
+  case FORMAT_JPEG:    name = ".jpg"; break;
+  case FORMAT_MPEG:    name = ".mpg"; break;
+  case FORMAT_PNG:     name = ".png"; break;
+  case FORMAT_PGF:     name = ".pgf"; break;
+  case FORMAT_PS:      name = ".ps"; break;
+  case FORMAT_EPS:     name = ".eps"; break;
+  case FORMAT_PDF:     name = ".pdf"; break;
+  case FORMAT_TEX:     name = ".tex"; break;
+  case FORMAT_SVG:     name = ".svg"; break;
+  case FORMAT_TIKZ:    name = ".tikz"; break;
+  case FORMAT_PPM:     name = ".ppm"; break;
+  case FORMAT_YUV:     name = ".yuv"; break;
+  case FORMAT_BREP:    name = ".brep"; break;
+  case FORMAT_IGES:    name = ".iges"; break;
+  case FORMAT_STEP:    name = ".step"; break;
+  case FORMAT_NEU:     name = ".neu"; mesh = true; break;
+  default:             name = ""; break;
+  }
+  if(onlyMeshFormats && !mesh) return "";
+  return name;
 }
 
 std::string GetDefaultFileName(int format)
 {
   std::vector<std::string> split = SplitFileName(GModel::current()->getFileName());
   std::string name = split[0] + split[1];
-  switch(format){
-  case FORMAT_GEO:  name += ".geo_unrolled"; break;
-  case FORMAT_MSH:  name += ".msh"; break;
-  case FORMAT_POS:  name += ".pos"; break;
-  case FORMAT_OPT:  name += ".opt"; break;
-  case FORMAT_UNV:  name += ".unv"; break;
-  case FORMAT_VTK:  name += ".vtk"; break;
-  case FORMAT_STL:  name += ".stl"; break;
-  case FORMAT_CGNS: name += ".cgns"; break;
-  case FORMAT_MED:  name += ".med"; break;
-  case FORMAT_RMED: name += ".rmed"; break;
-  case FORMAT_IR3:  name += ".ir3"; break;
-  case FORMAT_MESH: name += ".mesh"; break;
-  case FORMAT_MAIL: name += ".mail"; break;
-  case FORMAT_BDF:  name += ".bdf"; break;
-  case FORMAT_DIFF: name += ".diff"; break;
-  case FORMAT_INP:  name += ".inp"; break;
-  case FORMAT_CELUM:name += ".celum"; break;
-  case FORMAT_SU2:  name += ".su2"; break;
-  case FORMAT_P3D:  name += ".p3d"; break;
-  case FORMAT_VRML: name += ".wrl"; break;
-  case FORMAT_PLY2: name += ".ply2"; break;
-  case FORMAT_GIF:  name += ".gif"; break;
-  case FORMAT_JPEG: name += ".jpg"; break;
-  case FORMAT_MPEG: name += ".mpg"; break;
-  case FORMAT_PNG:  name += ".png"; break;
-  case FORMAT_PS:   name += ".ps"; break;
-  case FORMAT_EPS:  name += ".eps"; break;
-  case FORMAT_PDF:  name += ".pdf"; break;
-  case FORMAT_TEX:  name += ".tex"; break;
-  case FORMAT_SVG:  name += ".svg"; break;
-  case FORMAT_PPM:  name += ".ppm"; break;
-  case FORMAT_YUV:  name += ".yuv"; break;
-  case FORMAT_BREP: name += ".brep"; break;
-  case FORMAT_IGES: name += ".iges"; break;
-  case FORMAT_STEP: name += ".step"; break;
-  default: break;
+  std::string ext = GetDefaultFileExtension(format);
+  return name + ext;
+}
+
+std::string GetKnownFileFormats(bool onlyMeshFormats)
+{
+  std::string all = "auto, msh1, msh2, msh3, msh4";
+  for(int i = 1; i < 1000; i++){
+    std::string ext = GetDefaultFileExtension(i, onlyMeshFormats);
+    if(ext.size() > 1){
+      all += ", " + ext.substr(1);
+    }
   }
-  return name;
+  return all;
 }
 
 #if defined(HAVE_FLTK)
@@ -133,8 +177,8 @@ static PixelBuffer *GetCompositePixelBuffer(GLenum format, GLenum type)
   openglWindow *newg = 0;
 
   if(CTX::instance()->print.width > 0 || CTX::instance()->print.height > 0){
-    GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
-    GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
+    GLint width = FlGui::instance()->getCurrentOpenglWindow()->pixel_w();
+    GLint height = FlGui::instance()->getCurrentOpenglWindow()->pixel_h();
     if(CTX::instance()->print.width <= 0){
       double w = width * CTX::instance()->print.height / (double)height;
       width = (int)w;
@@ -166,8 +210,8 @@ static PixelBuffer *GetCompositePixelBuffer(GLenum format, GLenum type)
 
   PixelBuffer *buffer;
   if(newg || !CTX::instance()->print.compositeWindows){
-    GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
-    GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
+    GLint width = FlGui::instance()->getCurrentOpenglWindow()->pixel_w();
+    GLint height = FlGui::instance()->getCurrentOpenglWindow()->pixel_h();
     buffer = new PixelBuffer(width, height, format, type);
     buffer->fill(CTX::instance()->batch);
   }
@@ -186,11 +230,11 @@ static PixelBuffer *GetCompositePixelBuffer(GLenum format, GLenum type)
     std::vector<PixelBuffer*> buffers;
     for(unsigned int i = 0; i < g->gl.size(); i++){
       openglWindow::setLastHandled(g->gl[i]);
-      buffer = new PixelBuffer(g->gl[i]->w(), g->gl[i]->h(), format, type);
+      buffer = new PixelBuffer(g->gl[i]->pixel_w(), g->gl[i]->pixel_h(), format, type);
       buffer->fill(CTX::instance()->batch);
       buffers.push_back(buffer);
-      ww = std::max(ww, g->gl[i]->x() + g->gl[i]->w());
-      hh = std::max(hh, g->gl[i]->y() + g->gl[i]->h());
+      ww = std::max(ww, g->gl[i]->x() + g->gl[i]->pixel_w());
+      hh = std::max(hh, g->gl[i]->y() + g->gl[i]->pixel_h());
     }
     buffer = new PixelBuffer(ww, hh, format, type);
     for(unsigned int i = 0; i < g->gl.size(); i++){
@@ -220,7 +264,7 @@ void change_print_parameter(int frame)
   double v = first + frame * step * CTX::instance()->post.animStep;
   Msg::Info("Setting Print.Parameter = %g", v);
   opt_print_parameter(0, GMSH_SET | GMSH_GUI, v);
-  ParseString(CTX::instance()->print.parameterCommand);
+  ParseString(CTX::instance()->print.parameterCommand, true);
 }
 
 void CreateOutputFile(const std::string &fileName, int format,
@@ -240,7 +284,13 @@ void CreateOutputFile(const std::string &fileName, int format,
   switch (format) {
 
   case FORMAT_AUTO:
-    CreateOutputFile(name, GuessFileFormatFromFileName(name), false, false);
+    {
+      double version = 0.;
+      int format = GuessFileFormatFromFileName(name, &version);
+      if(format == FORMAT_MSH && version > 0.)
+        CTX::instance()->mesh.mshFileVersion = version;
+      CreateOutputFile(name, format, false, false);
+    }
     break;
 
   case FORMAT_OPT:
@@ -248,23 +298,27 @@ void CreateOutputFile(const std::string &fileName, int format,
     break;
 
   case FORMAT_MSH:
-    if(GModel::current()->getMeshPartitions().size() &&
-       CTX::instance()->mesh.mshFilePartitioned == 1)
+    if(GModel::current()->getNumPartitions() &&
+       CTX::instance()->mesh.partitionSplitMeshFiles){
+      std::vector<std::string> splitName = SplitFileName(name);
+      splitName[0] += splitName[1];
       GModel::current()->writePartitionedMSH
-        (name, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
-         CTX::instance()->mesh.saveParametric, CTX::instance()->mesh.scalingFactor);
-    else if(GModel::current()->getMeshPartitions().size() &&
-            CTX::instance()->mesh.mshFilePartitioned == 2)
-      GModel::current()->writeMSH
-        (name, CTX::instance()->mesh.mshFileVersion,
-         CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
-         CTX::instance()->mesh.saveParametric, CTX::instance()->mesh.scalingFactor,
-         0, -1000);
-    else
-      GModel::current()->writeMSH
-        (name, CTX::instance()->mesh.mshFileVersion,
+        (splitName[0], CTX::instance()->mesh.mshFileVersion,
          CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
          CTX::instance()->mesh.saveParametric, CTX::instance()->mesh.scalingFactor);
+    }
+    else{
+      GModel::current()->writeMSH
+        (name, CTX::instance()->mesh.mshFileVersion, CTX::instance()->mesh.binary,
+         CTX::instance()->mesh.saveAll, CTX::instance()->mesh.saveParametric,
+         CTX::instance()->mesh.scalingFactor);
+    }
+    if(GModel::current()->getNumPartitions() &&
+       CTX::instance()->mesh.partitionSaveTopologyFile){
+      std::vector<std::string> splitName = SplitFileName(name);
+      splitName[0] += splitName[1] + "_topology.pro";
+      GModel::current()->writePartitionedTopology(splitName[0]);
+    }
     break;
 
   case FORMAT_STL:
@@ -288,6 +342,11 @@ void CreateOutputFile(const std::string &fileName, int format,
        CTX::instance()->mesh.scalingFactor);
     break;
 
+  case FORMAT_TOCHNOG:
+    GModel::current()->writeTOCHNOG
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+    break;
+
   case FORMAT_VTK:
     GModel::current()->writeVTK
       (name, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
@@ -295,6 +354,12 @@ void CreateOutputFile(const std::string &fileName, int format,
        CTX::instance()->bigEndian);
     break;
 
+  case FORMAT_MATLAB:
+    GModel::current()->writeMATLAB
+      (name, CTX::instance()->mesh.binary, CTX::instance()->mesh.saveAll,
+       CTX::instance()->mesh.scalingFactor);
+    break;
+
   case FORMAT_MESH:
     GModel::current()->writeMESH
       (name, CTX::instance()->mesh.saveElementTagType,
@@ -331,6 +396,12 @@ void CreateOutputFile(const std::string &fileName, int format,
        CTX::instance()->mesh.scalingFactor);
     break;
 
+  case FORMAT_KEY:
+    GModel::current()->writeKEY
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.saveGroupsOfNodes,
+       CTX::instance()->mesh.scalingFactor);
+    break;
+
   case FORMAT_CELUM:
     GModel::current()->writeCELUM
       (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
@@ -360,10 +431,10 @@ void CreateOutputFile(const std::string &fileName, int format,
   case FORMAT_POS:
     GModel::current()->writePOS
       (name, CTX::instance()->print.posElementary,
-       CTX::instance()->print.posElement, CTX::instance()->print.posGamma,
-       CTX::instance()->print.posEta, CTX::instance()->print.posRho,
-       CTX::instance()->print.posDisto, CTX::instance()->mesh.saveAll,
-       CTX::instance()->mesh.scalingFactor);
+       CTX::instance()->print.posElement,
+       CTX::instance()->print.posSICN, CTX::instance()->print.posSIGE,
+       CTX::instance()->print.posGamma, CTX::instance()->print.posDisto,
+       CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
     break;
 
   case FORMAT_GEO:
@@ -379,6 +450,11 @@ void CreateOutputFile(const std::string &fileName, int format,
     GModel::current()->writeOCCSTEP(name);
     break;
 
+  case FORMAT_NEU:
+    GModel::current()->writeNEU
+      (name, CTX::instance()->mesh.saveAll, CTX::instance()->mesh.scalingFactor);
+    break;
+
 #if defined(HAVE_FLTK)
   case FORMAT_PPM:
   case FORMAT_YUV:
@@ -386,7 +462,10 @@ void CreateOutputFile(const std::string &fileName, int format,
   case FORMAT_JPEG:
   case FORMAT_PNG:
     {
-      if(!FlGui::available()) break;
+      if(!FlGui::available()){
+        Msg::Error("Creating '%s' requires a graphical interface context", name.c_str());
+        break;
+      }
 
       FILE *fp = Fopen(name.c_str(), "wb");
       if(!fp){
@@ -422,8 +501,12 @@ void CreateOutputFile(const std::string &fileName, int format,
   case FORMAT_EPS:
   case FORMAT_PDF:
   case FORMAT_SVG:
+  case FORMAT_TIKZ:
     {
-      if(!FlGui::available()) break;
+      if(!FlGui::available()){
+        Msg::Error("Creating '%s' requires a graphical interface context", name.c_str());
+        break;
+      }
 
       FILE *fp = Fopen(name.c_str(), "wb");
       if(!fp){
@@ -432,9 +515,9 @@ void CreateOutputFile(const std::string &fileName, int format,
         break;
       }
       std::string base = SplitFileName(name)[1];
-      GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
-      GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
-      GLint viewport[4] = {0, 0, width, height};
+      GLint width = FlGui::instance()->getCurrentOpenglWindow()->pixel_w();
+      GLint height = FlGui::instance()->getCurrentOpenglWindow()->pixel_h();
+      GLint pixel_viewport[4] = {0, 0, width, height};
 
       PixelBuffer buffer(width, height, GL_RGB, GL_FLOAT);
 
@@ -445,6 +528,7 @@ void CreateOutputFile(const std::string &fileName, int format,
         (format == FORMAT_PDF) ? GL2PS_PDF :
         (format == FORMAT_PS) ? GL2PS_PS :
         (format == FORMAT_SVG) ? GL2PS_SVG :
+        (format == FORMAT_TIKZ) ? GL2PS_PGF :
         GL2PS_EPS;
       int pssort =
         (CTX::instance()->print.epsQuality == 3) ? GL2PS_NO_SORT :
@@ -462,7 +546,7 @@ void CreateOutputFile(const std::string &fileName, int format,
       int res = GL2PS_OVERFLOW;
       while(res == GL2PS_OVERFLOW) {
         buffsize += 2048 * 2048;
-        gl2psBeginPage(base.c_str(), "Gmsh", viewport,
+        gl2psBeginPage(base.c_str(), "Gmsh", pixel_viewport,
                        psformat, pssort, psoptions, GL_RGBA, 0, NULL,
                        15, 20, 10, buffsize, fp, base.c_str());
         if(CTX::instance()->print.epsQuality == 0){
@@ -471,8 +555,8 @@ void CreateOutputFile(const std::string &fileName, int format,
           glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
           glMatrixMode(GL_PROJECTION);
           glLoadIdentity();
-          glOrtho((double)viewport[0], (double)viewport[2],
-                  (double)viewport[1], (double)viewport[3], -1., 1.);
+          glOrtho((double)pixel_viewport[0], (double)pixel_viewport[2],
+                  (double)pixel_viewport[1], (double)pixel_viewport[3], -1., 1.);
           glMatrixMode(GL_MODELVIEW);
           glLoadIdentity();
           glRasterPos2d(0, 0);
@@ -483,7 +567,7 @@ void CreateOutputFile(const std::string &fileName, int format,
           glLoadMatrixd(modelview);
         }
         else{
-          buffer.fill(CTX::instance()->batch);
+          drawContext::global()->drawCurrentOpenglWindow(true);
         }
         res = gl2psEndPage();
       }
@@ -494,7 +578,10 @@ void CreateOutputFile(const std::string &fileName, int format,
 
   case FORMAT_TEX:
     {
-      if(!FlGui::available()) break;
+      if(!FlGui::available()){
+        Msg::Error("Creating '%s' requires a graphical interface context", name.c_str());
+        break;
+      }
 
       FILE *fp = Fopen(name.c_str(), "w");
       if(!fp){
@@ -503,20 +590,19 @@ void CreateOutputFile(const std::string &fileName, int format,
         break;
       }
       std::string base = SplitFileName(name)[1];
-      GLint width = FlGui::instance()->getCurrentOpenglWindow()->w();
-      GLint height = FlGui::instance()->getCurrentOpenglWindow()->h();
-      GLint viewport[4] = {0, 0, width, height};
+      GLint width = FlGui::instance()->getCurrentOpenglWindow()->pixel_w();
+      GLint height = FlGui::instance()->getCurrentOpenglWindow()->pixel_h();
+      GLint pixel_viewport[4] = {0, 0, width, height};
       GLint buffsize = 0;
       int res = GL2PS_OVERFLOW;
       while(res == GL2PS_OVERFLOW) {
         buffsize += 2048 * 2048;
-        gl2psBeginPage(base.c_str(), "Gmsh", viewport,
+        gl2psBeginPage(base.c_str(), "Gmsh", pixel_viewport,
                        GL2PS_TEX, GL2PS_NO_SORT, GL2PS_NONE, GL_RGBA, 0, NULL,
                        0, 0, 0, buffsize, fp, base.c_str());
-        PixelBuffer buffer(width, height, GL_RGB, GL_UNSIGNED_BYTE);
         int oldtext = CTX::instance()->print.text;
         CTX::instance()->print.text = 1;
-        buffer.fill(CTX::instance()->batch);
+        drawContext::global()->drawCurrentOpenglWindow(true);
         CTX::instance()->print.text = oldtext;
         res = gl2psEndPage();
       }
@@ -524,10 +610,51 @@ void CreateOutputFile(const std::string &fileName, int format,
     }
     break;
 
+  case FORMAT_PGF:
+    {
+      if(!FlGui::available()){
+        Msg::Error("Creating '%s' requires a graphical interface context", name.c_str());
+        break;
+      }
+
+      // fill pixel buffer without colorbar and axes
+      int restoreGeneralAxis = (int) opt_general_axes(0, GMSH_GET, 0);
+      int restoreSmallAxis = (int) opt_general_small_axes(0, GMSH_GET, 0);
+      opt_general_axes(0, GMSH_SET, 0);
+      opt_general_small_axes(0, GMSH_SET, 0);
+      int num = -1; // id of the post view
+      int cnt = 0; // no of scales/colorbars active
+      for(unsigned int i = 0; i < opt_post_nb_views(0,GMSH_GET,0); i++) {
+        if(opt_view_visible(i, GMSH_GET, 0)) {
+          if (opt_view_show_scale(i, GMSH_GET, 0)) {
+            opt_view_show_scale(i, GMSH_SET, 0);
+            num = i; cnt++;
+          }
+        }
+      }
+      PixelBuffer *buffer = GetCompositePixelBuffer(GL_RGB, GL_UNSIGNED_BYTE);
+      drawContext *ctx = FlGui::instance()->getCurrentOpenglWindow()->getDrawContext();
+      GLint width = FlGui::instance()->getCurrentOpenglWindow()->pixel_w();
+      GLint height = FlGui::instance()->getCurrentOpenglWindow()->pixel_h();
+      GLint pixel_viewport[4] = {0, 0, width, height};
+      print_pgf(name, num, cnt, buffer, ctx->r, pixel_viewport, ctx->proj, ctx->model);
+      delete buffer;
+      // restore view
+      if(restoreGeneralAxis) opt_general_axes(0, GMSH_SET| GMSH_GUI, 1);
+      if(restoreSmallAxis) opt_general_small_axes(0, GMSH_SET | GMSH_GUI, 1);
+      if(cnt > 0) opt_view_show_scale(num, GMSH_SET, 1);
+    }
+    break;
+
 #if defined(HAVE_MPEG_ENCODE)
   case FORMAT_MPEG:
   case FORMAT_MPEG_PREVIEW:
     {
+      if(!FlGui::available()){
+        Msg::Error("Creating '%s' requires a graphical interface context", name.c_str());
+        break;
+      }
+
       std::string parFileName = CTX::instance()->homeDir + ".gmsh-mpeg_encode.par";
       FILE *fp = 0;
       if(format != FORMAT_MPEG_PREVIEW){
@@ -635,4 +762,3 @@ void CreateOutputFile(const std::string &fileName, int format,
   if(redraw) drawContext::global()->draw();
 #endif
 }
-
