@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -23,7 +23,7 @@
 // OF THIS SOFTWARE.
 //
 // Please report all bugs and problems to the public mailing list
-// <gmsh@geuz.org>.
+// <gmsh@onelab.info>.
 
 #ifndef _GMSH_SOCKET_H_
 #define _GMSH_SOCKET_H_
@@ -47,6 +47,7 @@
 #include <sys/un.h>
 #include <sys/time.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <netdb.h>
 #if defined(HAVE_NO_SOCKLEN_T)
 typedef int socklen_t;
@@ -85,6 +86,8 @@ class GmshSocket{
     GMSH_PARAMETER_UPDATE    = 32,
     GMSH_OPEN_PROJECT        = 33,
     GMSH_CLIENT_CHANGED      = 34,
+    GMSH_PARAMETER_WITHOUT_CHOICES = 35,
+    GMSH_PARAMETER_QUERY_WITHOUT_CHOICES = 36,
     GMSH_OPTION_1            = 100,
     GMSH_OPTION_2            = 101,
     GMSH_OPTION_3            = 102,
@@ -95,33 +98,37 @@ class GmshSocket{
   int _sock;
   // the socket name
   std::string _sockname;
+  // statistics
+  unsigned long int _sent, _received;
   // send some data over the socket
   int _SendData(const void *buffer, int bytes)
   {
     const char *buf = (const char *)buffer;
-    int sofar = 0;
-    int remaining = bytes;
+    long int sofar = 0;
+    long int remaining = bytes;
     do {
-      int len = send(_sock, buf + sofar, remaining, 0);
+      long int len = send(_sock, buf + sofar, remaining, 0);
       if(len < 0) return -1; // error
       sofar += len;
       remaining -= len;
     } while(remaining > 0);
+    _sent += bytes;
     return bytes;
   }
   // receive some data over the socket
   int _ReceiveData(void *buffer, int bytes)
   {
     char *buf = (char *)buffer;
-    int sofar = 0;
-    int remaining = bytes;
+    long int sofar = 0;
+    long int remaining = bytes;
     do {
-      int len = recv(_sock, buf + sofar, remaining, 0);
+      long int len = recv(_sock, buf + sofar, remaining, 0);
       if(len == 0) break; // we're done!
       if(len < 0) return -1; // error
       sofar += len;
       remaining -= len;
     } while(remaining > 0);
+    _received += bytes;
     return bytes;
   }
   // utility function to swap bytes in an array
@@ -146,7 +153,7 @@ class GmshSocket{
 #endif
   }
  public:
-  GmshSocket() : _sock(0)
+  GmshSocket() : _sock(0), _sent(0), _received(0)
   {
 #if defined(WIN32) && !defined(__CYGWIN__)
     WSADATA wsaData;
@@ -247,6 +254,8 @@ class GmshSocket{
     shutdown(s, SHUT_RDWR);
 #endif
   }
+  unsigned long int SentBytes(){ return _sent; }
+  unsigned long int ReceivedBytes(){ return _received; }
 };
 
 class GmshClient : public GmshSocket {
@@ -278,6 +287,9 @@ class GmshClient : public GmshSocket {
       // TCP/IP socket
       _sock = socket(AF_INET, SOCK_STREAM, 0);
       if(_sock < 0) return -1;
+      char one = 1;
+      // disable Nagle's algorithm (very slow for many small messages)
+      setsockopt(_sock, IPPROTO_TCP, TCP_NODELAY, &one, sizeof(one));
       // try to connect socket to host:port
       const char *port = strstr(sockname, ":");
       int portno = atoi(port + 1);
@@ -330,9 +342,9 @@ class GmshServer : public GmshSocket{
   virtual int NonBlockingWait(double waitint, double timeout, int socket=-1) = 0;
   // start the client by launching "exe args" (args is supposed to contain
   // '%s' where the socket name should appear)
-  int Start(const char *exe, const char *args, const char *sockname, double timeout)
+  int Start(const std::string &exe, const std::string &args, const std::string &sockname,
+            double timeout)
   {
-    if(!sockname) throw "Invalid (null) socket name";
     _sockname = sockname;
     int tmpsock;
     if(strstr(_sockname.c_str(), "/") || strstr(_sockname.c_str(), "\\") ||
@@ -368,6 +380,9 @@ class GmshServer : public GmshSocket{
       _portno = atoi(port + 1);
       // create a socket
       tmpsock = socket(AF_INET, SOCK_STREAM, 0);
+      char one = 1;
+      setsockopt(tmpsock, IPPROTO_TCP, TCP_NODELAY, &one, sizeof(one));
+
 #if !defined(WIN32) || defined(__CYGWIN__)
       if(tmpsock < 0)
 #else
@@ -395,9 +410,9 @@ class GmshServer : public GmshSocket{
       }
     }
 
-    if((exe && strlen(exe)) || (args && strlen(args))){
+    if(exe.size() || args.size()){
       char s[1024];
-      sprintf(s, args, _sockname.c_str());
+      sprintf(s, args.c_str(), _sockname.c_str());
       NonBlockingSystemCall(exe, s); // starts the solver
     }
     else{
@@ -435,9 +450,10 @@ class GmshServer : public GmshSocket{
       struct sockaddr_in from_in;
       socklen_t len = sizeof(from_in);
       _sock = accept(tmpsock, (struct sockaddr *)&from_in, &len);
+      char one = 1;
+      setsockopt(_sock, IPPROTO_TCP, TCP_NODELAY, &one, sizeof(one));
     }
     CloseSocket(tmpsock);
-
     if(_sock < 0)
       throw "Socket accept failed";
     return _sock;
