@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -13,11 +13,9 @@ void *Malloc(size_t size)
 {
   void *ptr;
 
-  if(!size)
-    return (NULL);
+  if(!size) return (NULL);
   ptr = malloc(size);
-  if(ptr == NULL)
-    Msg::Fatal("Out of memory (buy some more RAM!)");
+  if(ptr == NULL) Msg::Fatal("Out of memory (buy some more RAM!)");
   return (ptr);
 }
 
@@ -25,27 +23,22 @@ void *Calloc(size_t num, size_t size)
 {
   void *ptr;
 
-  if(!size)
-    return (NULL);
+  if(!size) return (NULL);
   ptr = calloc(num, size);
-  if(ptr == NULL)
-    Msg::Fatal("Out of memory (buy some more RAM!)");
+  if(ptr == NULL) Msg::Fatal("Out of memory (buy some more RAM!)");
   return (ptr);
 }
 
 void *Realloc(void *ptr, size_t size)
 {
-  if(!size)
-    return (NULL);
+  if(!size) return (NULL);
   ptr = realloc(ptr, size);
-  if(ptr == NULL)
-    Msg::Fatal("Out of memory (buy some more RAM!)");
+  if(ptr == NULL) Msg::Fatal("Out of memory (buy some more RAM!)");
   return (ptr);
 }
 
 void Free(void *ptr)
 {
-  if(ptr == NULL)
-    return;
+  if(ptr == NULL) return;
   free(ptr);
 }
