@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
@@ -9,21 +9,21 @@
 #include <string>
 #include "ColorTable.h"
 
-#define GMSH_SET         (1<<0)
-#define GMSH_GET         (1<<1)
-#define GMSH_GUI         (1<<2)
-#define GMSH_SET_DEFAULT (1<<3)
-#define GMSH_GET_DEFAULT (1<<4)
+#define GMSH_SET (1 << 0)
+#define GMSH_GET (1 << 1)
+#define GMSH_GUI (1 << 2)
+#define GMSH_SET_DEFAULT (1 << 3)
+#define GMSH_GET_DEFAULT (1 << 4)
 
-#define GMSH_SESSIONRC (1<<0)
-#define GMSH_OPTIONSRC (1<<1)
-#define GMSH_FULLRC    (1<<2)
+#define GMSH_SESSIONRC (1 << 0)
+#define GMSH_OPTIONSRC (1 << 1)
+#define GMSH_FULLRC (1 << 2)
 
 // action is a combination of GMSH_SET, GMSH_GET, GMSH_GUI
 
-#define OPT_ARGS_STR   int num, int action, std::string val
-#define OPT_ARGS_NUM   int num, int action, double val
-#define OPT_ARGS_COL   int num, int action, unsigned int val
+#define OPT_ARGS_STR int num, int action, std::string val
+#define OPT_ARGS_NUM int num, int action, double val
+#define OPT_ARGS_COL int num, int action, unsigned int val
 
 // STRINGS
 
@@ -35,6 +35,8 @@ std::string opt_general_axes_format1(OPT_ARGS_STR);
 std::string opt_general_axes_format2(OPT_ARGS_STR);
 std::string opt_general_background_image_filename(OPT_ARGS_STR);
 std::string opt_general_display(OPT_ARGS_STR);
+std::string opt_general_version(OPT_ARGS_STR);
+std::string opt_general_build_options(OPT_ARGS_STR);
 std::string opt_general_filename(OPT_ARGS_STR);
 std::string opt_general_executable_filename(OPT_ARGS_STR);
 std::string opt_general_default_filename(OPT_ARGS_STR);
@@ -58,6 +60,11 @@ std::string opt_general_gui_theme(OPT_ARGS_STR);
 std::string opt_general_graphics_font(OPT_ARGS_STR);
 std::string opt_general_graphics_font_title(OPT_ARGS_STR);
 std::string opt_general_graphics_font_engine(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_point_command(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_curve_command(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_surface_command(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_volume_command(OPT_ARGS_STR);
+std::string opt_geometry_occ_target_unit(OPT_ARGS_STR);
 std::string opt_solver_socket_name(OPT_ARGS_STR);
 std::string opt_solver_name(OPT_ARGS_STR);
 std::string opt_solver_name0(OPT_ARGS_STR);
@@ -70,6 +77,17 @@ std::string opt_solver_name6(OPT_ARGS_STR);
 std::string opt_solver_name7(OPT_ARGS_STR);
 std::string opt_solver_name8(OPT_ARGS_STR);
 std::string opt_solver_name9(OPT_ARGS_STR);
+std::string opt_solver_extension(OPT_ARGS_STR);
+std::string opt_solver_extension0(OPT_ARGS_STR);
+std::string opt_solver_extension1(OPT_ARGS_STR);
+std::string opt_solver_extension2(OPT_ARGS_STR);
+std::string opt_solver_extension3(OPT_ARGS_STR);
+std::string opt_solver_extension4(OPT_ARGS_STR);
+std::string opt_solver_extension5(OPT_ARGS_STR);
+std::string opt_solver_extension6(OPT_ARGS_STR);
+std::string opt_solver_extension7(OPT_ARGS_STR);
+std::string opt_solver_extension8(OPT_ARGS_STR);
+std::string opt_solver_extension9(OPT_ARGS_STR);
 std::string opt_solver_executable(OPT_ARGS_STR);
 std::string opt_solver_executable0(OPT_ARGS_STR);
 std::string opt_solver_executable1(OPT_ARGS_STR);
@@ -94,10 +112,12 @@ std::string opt_solver_remote_login8(OPT_ARGS_STR);
 std::string opt_solver_remote_login9(OPT_ARGS_STR);
 std::string opt_solver_octave_interpreter(OPT_ARGS_STR);
 std::string opt_solver_python_interpreter(OPT_ARGS_STR);
-std::string opt_post_graph_point_command(OPT_ARGS_STR);
+std::string opt_post_double_clicked_graph_point_command(OPT_ARGS_STR);
 std::string opt_view_name(OPT_ARGS_STR);
 std::string opt_view_format(OPT_ARGS_STR);
 std::string opt_view_filename(OPT_ARGS_STR);
+std::string opt_view_double_clicked_command(OPT_ARGS_STR);
+std::string opt_view_group(OPT_ARGS_STR);
 std::string opt_view_axes_label0(OPT_ARGS_STR);
 std::string opt_view_axes_label1(OPT_ARGS_STR);
 std::string opt_view_axes_label2(OPT_ARGS_STR);
@@ -123,6 +143,8 @@ std::string opt_print_parameter_command(OPT_ARGS_STR);
 // NUMBERS
 
 double opt_general_initial_context(OPT_ARGS_NUM);
+double opt_general_show_options_on_startup(OPT_ARGS_NUM);
+double opt_general_show_messages_on_startup(OPT_ARGS_NUM);
 double opt_general_fontsize(OPT_ARGS_NUM);
 double opt_general_gui_color_scheme(OPT_ARGS_NUM);
 double opt_general_graphics_fontsize(OPT_ARGS_NUM);
@@ -139,6 +161,7 @@ double opt_general_polygon_offset_always(OPT_ARGS_NUM);
 double opt_general_polygon_offset_factor(OPT_ARGS_NUM);
 double opt_general_polygon_offset_units(OPT_ARGS_NUM);
 double opt_general_system_menu_bar(OPT_ARGS_NUM);
+double opt_general_meshdiscrete(OPT_ARGS_NUM);
 double opt_general_message_size(OPT_ARGS_NUM);
 double opt_general_message_fontsize(OPT_ARGS_NUM);
 double opt_general_detached_menu(OPT_ARGS_NUM);
@@ -214,6 +237,7 @@ double opt_general_confirm_overwrite(OPT_ARGS_NUM);
 double opt_general_orthographic(OPT_ARGS_NUM);
 double opt_general_mouse_selection(OPT_ARGS_NUM);
 double opt_general_mouse_hover_meshes(OPT_ARGS_NUM);
+double opt_general_mouse_invert_zoom(OPT_ARGS_NUM);
 double opt_general_draw_bounding_box(OPT_ARGS_NUM);
 double opt_general_draw_oriented_bounding_box(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
@@ -263,6 +287,9 @@ double opt_general_zoom_factor(OPT_ARGS_NUM);
 double opt_general_expert_mode(OPT_ARGS_NUM);
 double opt_general_stereo_mode(OPT_ARGS_NUM);
 double opt_general_camera_mode(OPT_ARGS_NUM);
+#if defined(HAVE_VISUDEV)
+double opt_general_heavy_visualization(OPT_ARGS_NUM);
+#endif
 double opt_general_eye_sep_ratio(OPT_ARGS_NUM);
 double opt_general_focallength_ratio(OPT_ARGS_NUM);
 double opt_general_camera_aperture(OPT_ARGS_NUM);
@@ -323,6 +350,7 @@ double opt_general_light50(OPT_ARGS_NUM);
 double opt_general_light51(OPT_ARGS_NUM);
 double opt_general_light52(OPT_ARGS_NUM);
 double opt_general_light53(OPT_ARGS_NUM);
+double opt_general_num_threads(OPT_ARGS_NUM);
 double opt_geometry_transform(OPT_ARGS_NUM);
 double opt_geometry_transform00(OPT_ARGS_NUM);
 double opt_geometry_transform01(OPT_ARGS_NUM);
@@ -337,35 +365,39 @@ double opt_geometry_offset0(OPT_ARGS_NUM);
 double opt_geometry_offset1(OPT_ARGS_NUM);
 double opt_geometry_offset2(OPT_ARGS_NUM);
 double opt_geometry_auto_coherence(OPT_ARGS_NUM);
-double opt_geometry_hide_compounds(OPT_ARGS_NUM);
 double opt_geometry_oriented_physicals(OPT_ARGS_NUM);
 double opt_geometry_highlight_orphans(OPT_ARGS_NUM);
 double opt_geometry_tolerance(OPT_ARGS_NUM);
+double opt_geometry_tolerance_boolean(OPT_ARGS_NUM);
 double opt_geometry_normals(OPT_ARGS_NUM);
 double opt_geometry_tangents(OPT_ARGS_NUM);
 double opt_geometry_points(OPT_ARGS_NUM);
-double opt_geometry_lines(OPT_ARGS_NUM);
+double opt_geometry_curves(OPT_ARGS_NUM);
 double opt_geometry_surfaces(OPT_ARGS_NUM);
 double opt_geometry_volumes(OPT_ARGS_NUM);
 double opt_geometry_points_num(OPT_ARGS_NUM);
-double opt_geometry_lines_num(OPT_ARGS_NUM);
+double opt_geometry_curves_num(OPT_ARGS_NUM);
 double opt_geometry_surfaces_num(OPT_ARGS_NUM);
 double opt_geometry_volumes_num(OPT_ARGS_NUM);
 double opt_geometry_label_type(OPT_ARGS_NUM);
 double opt_geometry_point_size(OPT_ARGS_NUM);
 double opt_geometry_point_sel_size(OPT_ARGS_NUM);
 double opt_geometry_point_type(OPT_ARGS_NUM);
-double opt_geometry_line_width(OPT_ARGS_NUM);
-double opt_geometry_line_sel_width(OPT_ARGS_NUM);
-double opt_geometry_line_type(OPT_ARGS_NUM);
+double opt_geometry_reparam_on_face_robust(OPT_ARGS_NUM);
+double opt_geometry_curve_width(OPT_ARGS_NUM);
+double opt_geometry_curve_sel_width(OPT_ARGS_NUM);
+double opt_geometry_curve_type(OPT_ARGS_NUM);
 double opt_geometry_surface_type(OPT_ARGS_NUM);
 double opt_geometry_light(OPT_ARGS_NUM);
 double opt_geometry_light_two_side(OPT_ARGS_NUM);
+double opt_geometry_occ_auto_fix(OPT_ARGS_NUM);
+double opt_geometry_occ_disable_stl(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_degenerated(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_small_edges(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_small_faces(OPT_ARGS_NUM);
 double opt_geometry_occ_sew_faces(OPT_ARGS_NUM);
-double opt_geometry_occ_connect_faces(OPT_ARGS_NUM);
+double opt_geometry_occ_parallel(OPT_ARGS_NUM);
+double opt_geometry_occ_boolean_preserve_numbering(OPT_ARGS_NUM);
 double opt_geometry_occ_scaling(OPT_ARGS_NUM);
 double opt_geometry_old_circle(OPT_ARGS_NUM);
 double opt_geometry_old_newreg(OPT_ARGS_NUM);
@@ -379,13 +411,16 @@ double opt_geometry_snap1(OPT_ARGS_NUM);
 double opt_geometry_snap2(OPT_ARGS_NUM);
 double opt_geometry_clip(OPT_ARGS_NUM);
 double opt_geometry_copy_meshing_method(OPT_ARGS_NUM);
-double opt_geometry_copy_display_attributes(OPT_ARGS_NUM);
+double opt_geometry_double_clicked_entity_tag(OPT_ARGS_NUM);
 double opt_geometry_exact_extrusion(OPT_ARGS_NUM);
 double opt_geometry_match_geom_and_mesh(OPT_ARGS_NUM);
+double opt_geometry_match_mesh_scale_factor(OPT_ARGS_NUM);
+double opt_geometry_match_mesh_tolerance(OPT_ARGS_NUM);
 double opt_mesh_label_sampling(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
+double opt_mesh_optimize_threshold(OPT_ARGS_NUM);
 double opt_mesh_optimize_netgen(OPT_ARGS_NUM);
-double opt_mesh_remove_4_triangles(OPT_ARGS_NUM);
+double opt_mesh_old_refinement(OPT_ARGS_NUM);
 double opt_mesh_refine_steps(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
 double opt_mesh_num_sub_edges(OPT_ARGS_NUM);
@@ -402,6 +437,7 @@ double opt_mesh_lc_from_points(OPT_ARGS_NUM);
 double opt_mesh_lc_extend_from_boundary(OPT_ARGS_NUM);
 double opt_mesh_lc_integration_precision(OPT_ARGS_NUM);
 double opt_mesh_rand_factor(OPT_ARGS_NUM);
+double opt_mesh_rand_factor3d(OPT_ARGS_NUM);
 double opt_mesh_quality_inf(OPT_ARGS_NUM);
 double opt_mesh_quality_sup(OPT_ARGS_NUM);
 double opt_mesh_quality_type(OPT_ARGS_NUM);
@@ -416,6 +452,7 @@ double opt_mesh_tetrahedra(OPT_ARGS_NUM);
 double opt_mesh_hexahedra(OPT_ARGS_NUM);
 double opt_mesh_prisms(OPT_ARGS_NUM);
 double opt_mesh_pyramids(OPT_ARGS_NUM);
+double opt_mesh_trihedra(OPT_ARGS_NUM);
 double opt_mesh_surfaces_edges(OPT_ARGS_NUM);
 double opt_mesh_surfaces_faces(OPT_ARGS_NUM);
 double opt_mesh_volumes_edges(OPT_ARGS_NUM);
@@ -427,7 +464,6 @@ double opt_mesh_volumes_num(OPT_ARGS_NUM);
 double opt_mesh_point_size(OPT_ARGS_NUM);
 double opt_mesh_point_type(OPT_ARGS_NUM);
 double opt_mesh_line_width(OPT_ARGS_NUM);
-double opt_mesh_reverse_all_normals(OPT_ARGS_NUM);
 double opt_mesh_smooth_normals(OPT_ARGS_NUM);
 double opt_mesh_smooth_ratio(OPT_ARGS_NUM);
 double opt_mesh_angle_smooth_normals(OPT_ARGS_NUM);
@@ -438,29 +474,39 @@ double opt_mesh_light_two_side(OPT_ARGS_NUM);
 double opt_mesh_file_format(OPT_ARGS_NUM);
 double opt_mesh_newton_convergence_test_xyz(OPT_ARGS_NUM);
 double opt_mesh_msh_file_version(OPT_ARGS_NUM);
-double opt_mesh_msh_file_partitioned(OPT_ARGS_NUM);
+double opt_mesh_partition_split_mesh_files(OPT_ARGS_NUM);
+double opt_mesh_partition_save_topology_file(OPT_ARGS_NUM);
+double opt_mesh_partition_num(OPT_ARGS_NUM);
+double opt_mesh_partition_metis_algorithm(OPT_ARGS_NUM);
+double opt_mesh_partition_metis_edge_matching(OPT_ARGS_NUM);
+double opt_mesh_partition_metis_refinement_algorithm(OPT_ARGS_NUM);
 double opt_mesh_partition_hex_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_pri_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_pyr_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_trih_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_qua_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_tet_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_tri_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_create_topology(OPT_ARGS_NUM);
+double opt_mesh_partition_create_physicals(OPT_ARGS_NUM);
+double opt_mesh_partition_create_ghost_cells(OPT_ARGS_NUM);
+double opt_mesh_partition_old_style_msh2(OPT_ARGS_NUM);
 double opt_mesh_binary(OPT_ARGS_NUM);
-double opt_mesh_bunin(OPT_ARGS_NUM);
-double opt_mesh_lloyd(OPT_ARGS_NUM);
 double opt_mesh_smooth_cross_field(OPT_ARGS_NUM);
 double opt_mesh_bdf_field_format(OPT_ARGS_NUM);
+double opt_mesh_boundary_layer_fan_points(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
 double opt_mesh_algo2d(OPT_ARGS_NUM);
 double opt_mesh_algo3d(OPT_ARGS_NUM);
 double opt_mesh_algo_recombine(OPT_ARGS_NUM);
 double opt_mesh_recombine_all(OPT_ARGS_NUM);
 double opt_mesh_recombine3d_all(OPT_ARGS_NUM);
+double opt_mesh_recombine3d_level(OPT_ARGS_NUM);
+double opt_mesh_recombine3d_conformity(OPT_ARGS_NUM);
 double opt_mesh_flexible_transfinite(OPT_ARGS_NUM);
 double opt_mesh_do_recombination_test(OPT_ARGS_NUM);
 double opt_mesh_recombination_test_start(OPT_ARGS_NUM);
 double opt_mesh_recombination_no_greedy_strat(OPT_ARGS_NUM);
-double opt_mesh_recombination_new_strat(OPT_ARGS_NUM);
 double opt_mesh_remesh_algo(OPT_ARGS_NUM);
 double opt_mesh_remesh_param(OPT_ARGS_NUM);
 double opt_mesh_algo_subdivide(OPT_ARGS_NUM);
@@ -479,12 +525,14 @@ double opt_mesh_second_order_experimental(OPT_ARGS_NUM);
 double opt_mesh_second_order_linear(OPT_ARGS_NUM);
 double opt_mesh_second_order_incomplete(OPT_ARGS_NUM);
 double opt_mesh_cgns_import_order(OPT_ARGS_NUM);
+double opt_mesh_cgns_construct_topology(OPT_ARGS_NUM);
 double opt_mesh_dual(OPT_ARGS_NUM);
 double opt_mesh_voronoi(OPT_ARGS_NUM);
 double opt_mesh_draw_skin_only(OPT_ARGS_NUM);
 double opt_mesh_save_all(OPT_ARGS_NUM);
 double opt_mesh_save_element_tag_type(OPT_ARGS_NUM);
 double opt_mesh_save_parametric(OPT_ARGS_NUM);
+double opt_mesh_save_topology(OPT_ARGS_NUM);
 double opt_mesh_save_groups_of_nodes(OPT_ARGS_NUM);
 double opt_mesh_color_carousel(OPT_ARGS_NUM);
 double opt_mesh_switch_elem_tags(OPT_ARGS_NUM);
@@ -496,33 +544,19 @@ double opt_mesh_nb_tetrahedra(OPT_ARGS_NUM);
 double opt_mesh_nb_hexahedra(OPT_ARGS_NUM);
 double opt_mesh_nb_prisms(OPT_ARGS_NUM);
 double opt_mesh_nb_pyramids(OPT_ARGS_NUM);
+double opt_mesh_nb_trihedra(OPT_ARGS_NUM);
 double opt_mesh_cpu_time(OPT_ARGS_NUM);
-double opt_mesh_partition_partitioner(OPT_ARGS_NUM);
-double opt_mesh_partition_num(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_global_method(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_architecture(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_ndims_tot(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_mesh_dims1(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_mesh_dims2(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_mesh_dims3(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_local_method(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_eigensolver(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_vmax(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_nsection(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_eigtol(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_seed(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_refine_partition(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_internal_vertices(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_refine_map(OPT_ARGS_NUM);
-double opt_mesh_partition_chaco_terminal_propogation(OPT_ARGS_NUM);
-double opt_mesh_partition_metis_algorithm(OPT_ARGS_NUM);
-double opt_mesh_partition_metis_edge_matching(OPT_ARGS_NUM);
-double opt_mesh_partition_metis_refine_algorithm(OPT_ARGS_NUM);
 double opt_mesh_clip(OPT_ARGS_NUM);
-double opt_mesh_ignore_part_bound(OPT_ARGS_NUM);
+double opt_mesh_ignore_periodicity(OPT_ARGS_NUM);
+double opt_mesh_preserve_numbering_msh2(OPT_ARGS_NUM);
+double opt_mesh_max_num_threads_1d(OPT_ARGS_NUM);
+double opt_mesh_max_num_threads_2d(OPT_ARGS_NUM);
+double opt_mesh_max_num_threads_3d(OPT_ARGS_NUM);
+double opt_mesh_angle_tolerance_facet_overlap(OPT_ARGS_NUM);
 double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_timeout(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
+double opt_solver_auto_load_database(OPT_ARGS_NUM);
 double opt_solver_auto_save_database(OPT_ARGS_NUM);
 double opt_solver_auto_archive_output_files(OPT_ARGS_NUM);
 double opt_solver_auto_check(OPT_ARGS_NUM);
@@ -542,8 +576,12 @@ double opt_post_plugins(OPT_ARGS_NUM);
 double opt_post_nb_views(OPT_ARGS_NUM);
 double opt_post_file_format(OPT_ARGS_NUM);
 double opt_post_force_node_data(OPT_ARGS_NUM);
-double opt_post_graph_point_x(OPT_ARGS_NUM);
-double opt_post_graph_point_y(OPT_ARGS_NUM);
+double opt_post_force_element_data(OPT_ARGS_NUM);
+double opt_post_save_mesh(OPT_ARGS_NUM);
+double opt_post_save_interpolation_matrices(OPT_ARGS_NUM);
+double opt_post_double_clicked_graph_point_x(OPT_ARGS_NUM);
+double opt_post_double_clicked_graph_point_y(OPT_ARGS_NUM);
+double opt_post_double_clicked_view(OPT_ARGS_NUM);
 double opt_view_nb_timestep(OPT_ARGS_NUM);
 double opt_view_nb_non_empty_timestep(OPT_ARGS_NUM);
 double opt_view_timestep(OPT_ARGS_NUM);
@@ -643,6 +681,7 @@ double opt_view_draw_tetrahedra(OPT_ARGS_NUM);
 double opt_view_draw_hexahedra(OPT_ARGS_NUM);
 double opt_view_draw_prisms(OPT_ARGS_NUM);
 double opt_view_draw_pyramids(OPT_ARGS_NUM);
+double opt_view_draw_trihedra(OPT_ARGS_NUM);
 double opt_view_draw_scalars(OPT_ARGS_NUM);
 double opt_view_draw_vectors(OPT_ARGS_NUM);
 double opt_view_draw_tensors(OPT_ARGS_NUM);
@@ -669,6 +708,7 @@ double opt_view_component_map5(OPT_ARGS_NUM);
 double opt_view_component_map6(OPT_ARGS_NUM);
 double opt_view_component_map7(OPT_ARGS_NUM);
 double opt_view_component_map8(OPT_ARGS_NUM);
+double opt_view_closed(OPT_ARGS_NUM);
 double opt_print_file_format(OPT_ARGS_NUM);
 double opt_print_eps_compress(OPT_ARGS_NUM);
 double opt_print_eps_ps3shading(OPT_ARGS_NUM);
@@ -685,7 +725,8 @@ double opt_print_pos_elementary(OPT_ARGS_NUM);
 double opt_print_pos_element(OPT_ARGS_NUM);
 double opt_print_pos_gamma(OPT_ARGS_NUM);
 double opt_print_pos_eta(OPT_ARGS_NUM);
-double opt_print_pos_rho(OPT_ARGS_NUM);
+double opt_print_pos_SICN(OPT_ARGS_NUM);
+double opt_print_pos_SIGE(OPT_ARGS_NUM);
 double opt_print_pos_disto(OPT_ARGS_NUM);
 double opt_print_gif_dither(OPT_ARGS_NUM);
 double opt_print_gif_sort(OPT_ARGS_NUM);
@@ -705,6 +746,10 @@ double opt_print_parameter(OPT_ARGS_NUM);
 double opt_print_parameter_first(OPT_ARGS_NUM);
 double opt_print_parameter_last(OPT_ARGS_NUM);
 double opt_print_parameter_steps(OPT_ARGS_NUM);
+double opt_print_x3d_compatibility(OPT_ARGS_NUM);
+double opt_print_x3d_transparency(OPT_ARGS_NUM);
+double opt_print_x3d_remove_inner_borders(OPT_ARGS_NUM);
+double opt_print_x3d_precision(OPT_ARGS_NUM);
 
 // COLORS
 
@@ -718,7 +763,7 @@ unsigned int opt_general_color_ambient_light(OPT_ARGS_COL);
 unsigned int opt_general_color_diffuse_light(OPT_ARGS_COL);
 unsigned int opt_general_color_specular_light(OPT_ARGS_COL);
 unsigned int opt_geometry_color_points(OPT_ARGS_COL);
-unsigned int opt_geometry_color_lines(OPT_ARGS_COL);
+unsigned int opt_geometry_color_curves(OPT_ARGS_COL);
 unsigned int opt_geometry_color_surfaces(OPT_ARGS_COL);
 unsigned int opt_geometry_color_volumes(OPT_ARGS_COL);
 unsigned int opt_geometry_color_selection(OPT_ARGS_COL);
@@ -728,8 +773,8 @@ unsigned int opt_geometry_color_highlight2(OPT_ARGS_COL);
 unsigned int opt_geometry_color_tangents(OPT_ARGS_COL);
 unsigned int opt_geometry_color_normals(OPT_ARGS_COL);
 unsigned int opt_geometry_color_projection(OPT_ARGS_COL);
-unsigned int opt_mesh_color_points(OPT_ARGS_COL);
-unsigned int opt_mesh_color_points_sup(OPT_ARGS_COL);
+unsigned int opt_mesh_color_nodes(OPT_ARGS_COL);
+unsigned int opt_mesh_color_nodes_sup(OPT_ARGS_COL);
 unsigned int opt_mesh_color_lines(OPT_ARGS_COL);
 unsigned int opt_mesh_color_triangles(OPT_ARGS_COL);
 unsigned int opt_mesh_color_quadrangles(OPT_ARGS_COL);
@@ -737,6 +782,7 @@ unsigned int opt_mesh_color_tetrahedra(OPT_ARGS_COL);
 unsigned int opt_mesh_color_hexahedra(OPT_ARGS_COL);
 unsigned int opt_mesh_color_prisms(OPT_ARGS_COL);
 unsigned int opt_mesh_color_pyramid(OPT_ARGS_COL);
+unsigned int opt_mesh_color_trihedron(OPT_ARGS_COL);
 unsigned int opt_mesh_color_tangents(OPT_ARGS_COL);
 unsigned int opt_mesh_color_normals(OPT_ARGS_COL);
 unsigned int opt_mesh_color_0(OPT_ARGS_COL);
@@ -767,6 +813,7 @@ unsigned int opt_view_color_tetrahedra(OPT_ARGS_COL);
 unsigned int opt_view_color_hexahedra(OPT_ARGS_COL);
 unsigned int opt_view_color_prisms(OPT_ARGS_COL);
 unsigned int opt_view_color_pyramids(OPT_ARGS_COL);
+unsigned int opt_view_color_trihedra(OPT_ARGS_COL);
 unsigned int opt_view_color_tangents(OPT_ARGS_COL);
 unsigned int opt_view_color_normals(OPT_ARGS_COL);
 unsigned int opt_view_color_text2d(OPT_ARGS_COL);
@@ -777,8 +824,8 @@ unsigned int opt_view_color_background2d(OPT_ARGS_COL);
 // Data structures and global functions
 
 typedef struct {
-  const char *str ;
-  int int1, int2, int3, int4 ;
+  const char *str;
+  int int1, int2, int3, int4;
 } StringX4Int;
 
 typedef struct {
@@ -793,7 +840,7 @@ typedef struct {
   int level;
   const char *str;
   double (*function)(int num, int action, double val);
-  double def ;
+  double def;
   const char *help;
 } StringXNumber;
 
@@ -812,15 +859,15 @@ void InitOptions(int num);
 void InitOptionsGUI(int num);
 void ReInitOptions(int num);
 void PrintOptions(int num, int level, int diff, int help, const char *filename,
-                  std::vector<std::string> *vec=0);
+                  std::vector<std::string> *vec = 0);
 void PrintOptionsDoc();
 
-bool StringOption(int action, const char *category, int num,
-                  const char *name, std::string &val);
-bool NumberOption(int action, const char *category, int num,
-                  const char *name, double &val);
-bool ColorOption(int action, const char *category, int num,
-                 const char *name, unsigned int &val);
+bool StringOption(int action, const char *category, int num, const char *name,
+                  std::string &val, bool warnIfUnknown = true);
+bool NumberOption(int action, const char *category, int num, const char *name,
+                  double &val, bool warnIfUnknown = true);
+bool ColorOption(int action, const char *category, int num, const char *name,
+                 unsigned int &val, bool warnIfUnknown = true);
 
 GmshColorTable *GetColorTable(int num);
 int GetColorForString(int alpha, const char *string, int *FlagError);
