@@ -1,19 +1,20 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _GMSH_EDGE_H_
 #define _GMSH_EDGE_H_
 
-#include "Geo.h"
 #include "GEdge.h"
 
-class gmshEdge : public GEdge {
- protected:
-  Curve *c; 
+class Curve;
 
- public:
+class gmshEdge : public GEdge {
+protected:
+  Curve *c;
+
+public:
   gmshEdge(GModel *model, Curve *edge, GVertex *v1, GVertex *v2);
   virtual ~gmshEdge() {}
   virtual Range<double> parBounds(int i) const;
@@ -22,13 +23,17 @@ class gmshEdge : public GEdge {
   virtual SVector3 firstDer(double par) const;
   virtual SVector3 secondDer(double par) const;
   ModelType getNativeType() const { return GmshModel; }
-  void * getNativePtr() const { return c; }
-  virtual std::string getAdditionalInfoString();
+  void *getNativePtr() const { return c; }
+  virtual std::string getAdditionalInfoString(bool multline = false);
   virtual int minimumMeshSegments() const;
   virtual int minimumDrawSegments() const;
   virtual void resetMeshAttributes();
   virtual SPoint2 reparamOnFace(const GFace *face, double epar, int dir) const;
   virtual void writeGEO(FILE *fp);
+  void discretize(double tol, std::vector<SPoint3> &dpts,
+                  std::vector<double> &ts);
+  virtual bool degenerate(int dim) const;
+  void resetNativePtr(Curve *edge, GVertex *v1, GVertex *v2);
 };
 
 #endif
